#!/bin/bash
#
#	Which platform are we packaging on Apple or Linux?
#
COMMAND=`echo $1 | tr "[:lower:]" "[:upper:]"` 
MACHTYPE=`echo $MACHTYPE | tr "[:lower:]" "[:upper:]"`
PLATFORM=`echo $MACHTYPE | grep -i apple`
#CURRENT_USER=`who | grep console | cut -d " " -f 1`
CALLING_USER=`whoami`
ARTWORK_USER=$USER

UnloadService()
{
	echo Stopping tvmobilisvcd...
	pushd /Library/LaunchDaemons > /dev/null
	if [ -f com.tvmobili.tvmobilisvcd.plist ]; then
		sudo launchctl unload com.tvmobili.tvmobilisvcd.plist
#		launchctl unload com.tvmobili.tvmobilisvcd.plist
	fi
	popd > /dev/null
}

LoadService()
{
	echo Starting tvmobilisvcd...
	pushd /Library/LaunchDaemons > /dev/null
	sudo launchctl load com.tvmobili.tvmobilisvcd.plist
#	launchctl load com.tvmobili.tvmobilisvcd.plist
	popd > /dev/null
}

UnloadArtworkApp()
{
	echo Stopping artwork helper...
	pushd /Library/LaunchAgents > /dev/null
	if [ -f com.tvmobili.artwork.plist ]; then
#		launchctl unload com.tvmobili.artwork.plist
		if [[ $COMMAND == "NEWVERSION" ]]; then
			su $ARTWORK_USER -c 'launchctl unload com.tvmobili.artwork.plist'
		else
			launchctl unload com.tvmobili.artwork.plist
		fi
	fi
	popd > /dev/null
}

LoadArtworkApp()
{
	echo Starting artwork helper...
	pushd /Library/LaunchAgents > /dev/null
#	launchctl load com.tvmobili.artwork.plist
	if [[ $COMMAND == "NEWVERSION" ]]; then
		su $ARTWORK_USER -c 'launchctl load com.tvmobili.artwork.plist'
	else
		launchctl load com.tvmobili.artwork.plist
	fi
	popd > /dev/null
}

if [[ $PLATFORM == $MACHTYPE ]]; then

	#
	#	Need to run as root otherwise the daemon will either fail to start or will be run
	#	under the wrong account
	#
	#	MWF 2/11/10
	#	Reversed this, could not restart the artwork handler by su'ing to the current user, got a broken socket error for some mad reason
	#	This script now must be called as the logged on user and the commands that need to be run as root are prefixed by sudo
	#
	#if [[ $CALLING_USER != "root" ]]; then
	#	echo
	#	echo "Must be run as root. Have you forgotten to prefix the command with sudo?"
	#	echo
	#	exit
	#fi
	if [[ $COMMAND != "NEWVERSION" ]]; then
		if [[ $CALLING_USER == "root" ]]; then
			echo
			echo "Must be run as a regular user. You will be prompted for the su password when required."
			echo
			exit
		fi
	fi

	#
	#	Need to run the artwork handler in the account of the logged on user. This is easyish to determine but
	#	is done in different ways depending on how the script is being run. If the script is being run from
	#	the installer (most likely) then $USER should contain the name of the logged on user. However, if the
	#	script is called interactively via sudo then $USER will have changed to root but $SUDO_USER should
	#	contain the name of the logged on user.
	#
	#	As I understand it, it is a known issue with LaunchAgents that upgrading, installing is a nightmare
	#	without getting the user to log out or reboot. This is the preferred way since there is no messing
	#	around with user accounts and calling launchctl (which incidentally does not work properly - calling
	#	this script interactively to start the agents/daemons will not work for the artwork handler - you will
	#	get an error 'launch_msg(): Socket is not connected' and the agent will not start). No matter, this is
	#	the least likely use of this script and in any case the app will launch properly next time the user
	#	logs on. It is more important that this script behaves correctly during an upgrade / install.
	#
	ARTWORK_USER=$USER
	if [[ $ARTWORK_USER == "root" ]]; then
		ARTWORK_USER=$SUDO_USER
	fi

	if [[ $COMMAND == "STOP" ]]; then	
		UnloadService
		UnloadArtworkApp
	fi

	if [[ $COMMAND == "START" ]]; then
		LoadService
		LoadArtworkApp
	fi

	if [[ $COMMAND == "RESTART" ]]; then
		UnloadArtworkApp
		UnloadService
		sleep 2
		LoadService
		LoadArtworkApp
	fi

	if [[ $COMMAND == "NEWVERSION" ]]; then
		UnloadArtworkApp
		UnloadService

		#
		#	Commented out as this is flawed! This doesn't take into account the case where there is a username key preset BUT
		#	its value is set to root!! Fix later
		#
		if [ ! -e /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.plist ] ; then
			cp /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.tvmobilisvcd.plist.root /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.plist
		fi
		if [ ! -e /Library/LaunchAgents/com.tvmobili.artwork.plist ] ; then
			cp /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.artwork.plist /Library/LaunchAgents/com.tvmobili.artwork.plist
		fi
		
		#
		#	Just in case, mop up any stragglers. It does appear that we have 1 too many qlmanager processes running but at this point
		#	however we do not know why. Unfortunately - the extra process is the 'right' process and is not controllable via launchagent
		# 	hence on a reinstall, we will need to kill it to pick up the new binary.
		#	The 'too many qlmanagers' issue is possibly due to a bug in StartProcess which possibly resulted in 2 processes being started
		#	(just because of the way it works). StartProcess launches a process to launch the actual required process but the first process
		#	does not terminate. This is just a theory at the moment but thought I'd document it here in case this gets investigated again.
		#
		#	Not sure this is the case anymore, think this was just a failure in the way we were starting the qlmanager i.e. we were starting
		#	it in the context of the root account during an upgrade which spawned the second instance of the process.
		#
		#ps -A | grep -i tvm | awk '$5 ~ /\/Applications\/TVMOBiLi.app\/Contents\/MacOS\/tvmobiliqlhandler/ { print "kill -9",$1;}' > /tvmql--XX.sh
		#chmod 777 /tvmql--XX.sh
		#/tvmql--XX.sh
		#rm -f tvmql--XX.sh
		sleep 2
		LoadService
		LoadArtworkApp
	fi

	#
	#	Switches to run the app in privileged mode (artwork handler ALWAYS runs unprivilieged)
	#
	if [[ $COMMAND == "PRIV" ]]; then
		UnloadService
		sudo cp /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.tvmobilisvcd.plist.root /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.plist
		sleep 2
		LoadService
	fi

	#
	#	Switches to run the app in normal, unprivileged mode (artwork handler ALWAYS runs unprivilieged)
	#
	if [[ $COMMAND == "UNPRIV" ]]; then
		UnloadService
		sudo cp /Applications/TVMOBiLi.app/Contents/MacOS/com.tvmobili.tvmobilisvcd.plist.tvmobili /Library/LaunchDaemons/com.tvmobili.tvmobilisvcd.plist
		if [ -e /Applications/TVMOBiLi.app/Contents/conf/tvmobiliuser.dat ]; then
			TVMUSER_ACC=`cat /Applications/TVMOBiLi.app/Contents/tvmobiliuser.dat`
			chown -R $TVMUSER_ACC:$TVMUSER_ACC /Applications/TVMOBiLi.app/Contents
		fi
		sleep 2
		LoadService
	fi

else

	if [[ $COMMAND == "PRIV" ]]; then
		/etc/init.d/tvmobilisvcd stop
		cp /etc/tvmobili/template-instance/conf/tvmobilisvcd.init.root /etc/init.d/tvmobilisvcd
		/etc/init.d/tvmobilisvcd start
	else
		if [[ $COMMAND == "UNPRIV" ]]; then
			/etc/init.d/tvmobilisvcd stop
			cp /etc/tvmobili/template-instance/conf/tvmobilisvcd.init.tvmobili /etc/init.d/tvmobilisvcd
			if [ -e /etc/tvmobili/tvmobiliuser.dat ]; then
				TVMUSER_ACC=`cat /etc/tvmobili/tvmobiliuser.dat`
				chown -R $TVMUSER_ACC:$TVMUSER_ACC /etc/tvmobili
			fi
			/etc/init.d/tvmobilisvcd start
		else
			/etc/init.d/tvmobilisvcd $COMMAND
		fi
	fi

fi
